using System;
using System.Linq;
using FireStarterModeller.BO;
using FireStarterModeller.Logic.Wizards;
using FireStarterModeller.Test.BO;
using FireStarterModeller.Test.BO.Fakes;
using Habanero.BO;
using NUnit.Framework;
using Rhino.Mocks;

namespace FireStarterModeller.Test.Logic
{
    [TestFixture]
    public class TestSolutionRenamer
    {
        #region Setup/Teardown

        [SetUp]
        public virtual void SetupTest()
        {
            TestUtilsLogic.SetupTest();
        }

        [TearDown]
        public virtual void TearDownTest()
        {
        }

        #endregion

        [TestFixtureSetUp]
        public virtual void TestFixtureSetup()
        {
            TestUtilsLogic.SetupFixture();
        }

        [Test]
        public void Test_Construct_WhenRelationshipPropertyNull_ShouldRaiseError()
        {
            //---------------Set up test pack-------------------
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            try
            {
                new SolutionRenamer(null);
                Assert.Fail("expected ArgumentNullException");
            }
                //---------------Test Result -----------------------
            catch (ArgumentNullException ex)
            {
                StringAssert.Contains("Value cannot be null", ex.Message);
                StringAssert.Contains("solution", ex.ParamName);
            }
        }

        [Test]
        public void Test_Construct_ShouldSetSolution()
        {
            //---------------Set up test pack-------------------
            var mockDMSolution = MockRepository.GenerateMock<DMSolution>(ConstructForFakes.True);
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            var solutionRenamer = new SolutionRenamer(mockDMSolution);
            //---------------Test Result -----------------------
            Assert.AreSame(mockDMSolution, solutionRenamer.Solution);
        }
        [Test]
        public void Test_RenameToStandardNames_WhenOneClass_ShouldCallRenameClass()
        {
            //---------------Set up test pack-------------------
            FakeDMSolution mockDMSolution = GetMockDMSolution();
            var mockDMClass = MockRepository.GenerateMock<FakeDMClass>();
            mockDMSolution.AllClasses.Add(mockDMClass);
            var solutionRenamer = new SolutionRenamer(mockDMSolution);
            //---------------Assert Precondition----------------
            Assert.Contains(mockDMClass, mockDMSolution.AllClasses);
            mockDMClass.AssertWasNotCalled(@class => @class.RenameClass());
            //---------------Execute Test ----------------------
            solutionRenamer.RenameToStandardNames();
            //---------------Test Result -----------------------
            mockDMClass.AssertWasCalled(@class => @class.RenameClass());
        }    
        [Test]
        public void Test_RenameToStandardNames_WhenTwoClass_ShouldCallRenameClass()
        {
            //---------------Set up test pack-------------------
            FakeDMSolution mockDMSolution = GetMockDMSolution();
            var mockDMClass = MockRepository.GenerateMock<FakeDMClass>();
            var mockDMClass2 = MockRepository.GenerateMock<FakeDMClass>();
            mockDMSolution.AllClasses.Add(mockDMClass);
            mockDMSolution.AllClasses.Add(mockDMClass2);
            var solutionRenamer = new SolutionRenamer(mockDMSolution);
            //---------------Assert Precondition----------------
            Assert.Contains(mockDMClass, mockDMSolution.AllClasses);
            Assert.Contains(mockDMClass2, mockDMSolution.AllClasses);
            mockDMClass.AssertWasNotCalled(@class => @class.RenameClass());
            mockDMClass2.AssertWasNotCalled(@class => @class.RenameClass());
            //---------------Execute Test ----------------------
            solutionRenamer.RenameToStandardNames();
            //---------------Test Result -----------------------
            mockDMClass.AssertWasCalled(@class => @class.RenameClass());
            mockDMClass2.AssertWasCalled(@class => @class.RenameClass());
        }

        private static FakeDMSolution GetMockDMSolution()
        {
            var mockDMSolution = MockRepository.GenerateMock<FakeDMSolution>();
            mockDMSolution.Stub(solution => solution.AllClasses).Return(new FakeBusinessObjectCollection<DMClass>());
            return mockDMSolution;
        }

/*
        [Test]
        public void Test_Delete_WhenNoRelationshipsAndNoLookups_ShouldMarkForDelete()
        {
            //If the Class is not involved in any Relationship and is not 
            // being used in Any Lookup Lists then it should be deleted.
            //---------------Set up test pack-------------------
            IDMSolution dmSolution = GetMockDMSolution();
            var solutionRenamer = new SolutionRenamer(dmSolution);
            //---------------Assert Precondition----------------
            Assert.IsEmpty(dmSolution.RelationshipsDM);
            Assert.IsEmpty(dmSolution.DMLookupListBusinessObjects);
            dmSolution.AssertWasNotCalled(@class => @class.MarkForDelete());
            //---------------Execute Test ----------------------
            solutionRenamer.Delete();
            //---------------Test Result -----------------------
            dmSolution.AssertWasCalled(@class => @class.MarkForDelete());
        }
        //This occurs because the ClassDef.XML has DeleteRelated Set to true
        [Test]
        public void Test_Delete_WhenHasRelationshipsAndNoLookups_ShouldMarkForDeleteAndMarkRelsForDelete()
        {
            //If the Class is involved in any Relationship and is not 
            // being used in Any Lookup Lists then it should be deleted along with its Relationships.
            //---------------Set up test pack-------------------
            IDMSolution dmSolution = new DMSolution();
            var relationship = new DMRelationship();
            dmSolution.RelationshipsDM.Add(relationship);
            var solutionRenamer = new SolutionRenamer(dmSolution);
            //---------------Assert Precondition----------------
            Assert.IsNotEmpty(dmSolution.RelationshipsDM);
            Assert.IsEmpty(dmSolution.DMLookupListBusinessObjects);
            Assert.IsFalse(relationship.Status.IsDeleted);
            //---------------Execute Test ----------------------
            solutionRenamer.Delete();
            //---------------Test Result -----------------------
            Assert.IsTrue(dmSolution.Status.IsDeleted);
            Assert.IsTrue(relationship.Status.IsDeleted);
        }
        //This occurs because the ClassDef.XML has DeleteRelated Set to true
        [Test]
        public void Test_Delete_WhenHasRelationshipsAndHasLookups_ShouldMarkLookupsForDelete()
        {
            //If the Class is involved in any Relationship and is not 
            // being used in Any Lookup Lists then it should be deleted along with its Relationships.
            //---------------Set up test pack-------------------
            IDMSolution dmSolution = new DMSolution();
            var relationship = new DMRelationship();
            dmSolution.RelationshipsDM.Add(relationship);

            var dmLookupListBusinessObject = new DMLookupListBusinessObject();
            dmSolution.DMLookupListBusinessObjects.Add(dmLookupListBusinessObject);

            var solutionRenamer = new SolutionRenamer(dmSolution);
            //---------------Assert Precondition----------------
            Assert.IsNotEmpty(dmSolution.RelationshipsDM);
            Assert.IsNotEmpty(dmSolution.DMLookupListBusinessObjects);
            Assert.IsFalse(dmLookupListBusinessObject.Status.IsDeleted);
            //---------------Execute Test ----------------------
            solutionRenamer.Delete();
            //---------------Test Result -----------------------
            Assert.IsTrue(dmLookupListBusinessObject.Status.IsDeleted);
        }

        [Test] 
        public void Test_AllRelationshipWithRelatedFKs_WhenHasOne_ShouldReturnRel()
        {
            //---------------Set up test pack-------------------
            IDMSolution dmSolution = GetMockDMSolution();
            DMRelationship mockRel = GetMockRel(true);
            dmSolution.RelationshipsDM.Add(mockRel);
            var mockRelRelBOHasFKFalse = GetMockRel(false);
            dmSolution.RelationshipsDM.Add(mockRelRelBOHasFKFalse);
            var solutionRenamer = new SolutionRenamer(dmSolution);
            //---------------Assert Precondition----------------
            Assert.IsTrue(mockRel.RelatedBOHasForeignKey);
            Assert.IsFalse(mockRelRelBOHasFKFalse.RelatedBOHasForeignKey);
            //---------------Execute Test ----------------------
            var allRelationshipWithRelatedFKs = solutionRenamer.AllRelationshipWithRelatedFKs;
            //---------------Test Result -----------------------
            Assert.AreEqual(1, allRelationshipWithRelatedFKs.Count());
            Assert.Contains(mockRel, allRelationshipWithRelatedFKs.ToList());
        }

        private static DMRelationship GetMockRel(bool relatedBOHasFK)
        {
            var mockRel = MockRepository.GenerateMock<DMRelationship>();
            mockRel.Stub(relationship => relationship.RelatedBOHasForeignKey).Return(relatedBOHasFK);
            return mockRel;
        }


        [Test]
        public void Test_AddRelationshipWithFKToBeDeleted()
        {
            //---------------Set up test pack-------------------
            IDMSolution dmSolution = GetMockDMSolution();
            var solutionRenamer = new SolutionRenamer(dmSolution);
            var dmRelationship = MockRepository.GenerateMock<IDMRelationship>();

            //---------------Assert Precondition----------------
            Assert.AreEqual(0, solutionRenamer.RelationshipsWithFKToBeDeleted.Count());
            //---------------Execute Test ----------------------
            solutionRenamer.AddRelationshipWithFKToBeDeleted(dmRelationship);
            //---------------Test Result -----------------------
            Assert.AreEqual(1, solutionRenamer.RelationshipsWithFKToBeDeleted.Count());
            Assert.AreSame(dmRelationship, solutionRenamer.RelationshipsWithFKToBeDeleted.First());
        }
        [Test]
        public void Test_Delete_WhenHasRelationshipWithRelatedBOFK_AndFKPropSelectedToDelete_ShouldDeleteFKProp()
        {
            //If when deleting the DMSolution the user selected to Delete the Related FKProp
            // the the Related FKProp Should Be Deleted
            //---------------Set up test pack-------------------
            IDMSolution dmSolution = GetMockDMSolution();
            var solutionRenamer = new SolutionRenamer(dmSolution);

            var dmRelationship = new FakeDMRelationship();
            dmRelationship.SetRelationshipProperties(new FakeBusinessObjectCollection<DMRelationshipProperty>());

            var relationshipProperty = new DMRelationshipProperty();
            var relatedProperty = new DMProperty();
            relationshipProperty.RelatedProperty = relatedProperty;
            dmRelationship.RelationshipProperties.Add(relationshipProperty);

            solutionRenamer.AddRelationshipWithFKToBeDeleted(dmRelationship);
            //---------------Assert Precondition----------------
            Assert.AreSame(dmRelationship, solutionRenamer.RelationshipsWithFKToBeDeleted.First());
            Assert.AreSame(relationshipProperty, dmRelationship.RelationshipProperties.First());
            Assert.IsFalse(relationshipProperty.Status.IsDeleted);
            Assert.IsFalse(relatedProperty.Status.IsDeleted);
            //---------------Execute Test ----------------------
            solutionRenamer.Delete();
            //---------------Test Result -----------------------
            Assert.IsTrue(relationshipProperty.Status.IsDeleted);
            Assert.IsTrue(relatedProperty.Status.IsDeleted);
        }
        [Test]
        public void Test_Delete_WhenHasTwoRelationshipWithRelatedBOFK_AndFKPropSelectedToDelete_ShouldDeleteFKProp()
        {
            //If when deleting the DMSolution the user selected to Delete the Related FKProp
            // the the Related FKProp Should Be Deleted
            //---------------Set up test pack-------------------
            IDMSolution dmSolution = GetMockDMSolution();
            var solutionRenamer = new SolutionRenamer(dmSolution);

            FakeDMRelationship dmRelationship = GetFakeRelationship();
            FakeDMRelationshipProperty relationshipProperty = AddRelProp(dmRelationship);
            FakeDMRelationship dmRelationship2 = GetFakeRelationship();
            FakeDMRelationshipProperty relationshipProperty2 = AddRelProp(dmRelationship2);

            solutionRenamer.AddRelationshipWithFKToBeDeleted(dmRelationship);
            solutionRenamer.AddRelationshipWithFKToBeDeleted(dmRelationship2);
            //---------------Assert Precondition----------------
            Assert.AreSame(dmRelationship, solutionRenamer.RelationshipsWithFKToBeDeleted.First());
            Assert.AreSame(relationshipProperty, dmRelationship.RelationshipProperties.First());
            Assert.IsFalse(relationshipProperty.Status.IsDeleted);
            Assert.AreEqual(2, solutionRenamer.RelationshipsWithFKToBeDeleted.Count);
            //---------------Execute Test ----------------------
            solutionRenamer.Delete();
            //---------------Test Result -----------------------
            Assert.IsTrue(relationshipProperty.Status.IsDeleted);
            Assert.IsTrue(relationshipProperty2.Status.IsDeleted);
        }

        private static FakeDMRelationshipProperty AddRelProp(FakeDMRelationship dmRelationship)
        {
            var relationshipProperty = new FakeDMRelationshipProperty();
            dmRelationship.RelationshipProperties.Add(relationshipProperty);
            return relationshipProperty;
        }

        private static FakeDMRelationship GetFakeRelationship()
        {
            var dmRelationship = new FakeDMRelationship();
            dmRelationship.SetRelationshipProperties(new FakeBusinessObjectCollection<DMRelationshipProperty>());
            return dmRelationship;
        }
        [Test]
        public void Test_AllBOLookupsWithPropsToBeDeleted_WhenHasOne_ShouldReturnRel()
        {
            //---------------Set up test pack-------------------
            IDMSolution dmSolution = GetMockDMSolution();
            
            var solutionRenamer = new SolutionRenamer(dmSolution);
            //---------------Assert Precondition----------------
            Assert.AreEqual(0, solutionRenamer.AllLookupLists.Count());
            //---------------Execute Test ----------------------
            dmSolution.DMLookupListBusinessObjects.Add(new DMLookupListBusinessObject());
            var allLookupLists = solutionRenamer.AllLookupLists;
            //---------------Test Result -----------------------
            Assert.AreEqual(1, allLookupLists.Count());
        }
        [Test]
        public void Test_AddBOLListWithLookupListPropsToBeDeleted()
        {
            //---------------Set up test pack-------------------
            IDMSolution dmSolution = GetMockDMSolution();
            var solutionRenamer = new SolutionRenamer(dmSolution);
            var lookupList = new DMLookupListBusinessObject();

            //---------------Assert Precondition----------------
            Assert.AreEqual(0, solutionRenamer.LookupListsWithPropsToBeDeleted.Count());
            //---------------Execute Test ----------------------
            solutionRenamer.AddLookupListsWithPropsToBeDeleted(lookupList);
            //---------------Test Result -----------------------
            Assert.AreEqual(1, solutionRenamer.LookupListsWithPropsToBeDeleted.Count());
            Assert.AreSame(lookupList, solutionRenamer.LookupListsWithPropsToBeDeleted.First());
        }
        [Test]
        public void Test_Delete_WhenHasLookupLists_ShouldDeleteLookupProp()
        {
            //If when deleting the DMSolution the user selected to Delete the Related FKProp
            // the the Related FKProp Should Be Deleted
            //---------------Set up test pack-------------------
            IDMSolution dmSolution = GetMockDMSolution();
            var solutionRenamer = new SolutionRenamer(dmSolution);


            var dmProperty = new DMProperty();
            var lookupList = new DMLookupListBusinessObject {DMProperty = dmProperty};
            solutionRenamer.AddLookupListsWithPropsToBeDeleted(lookupList);
            //---------------Assert Precondition----------------
            Assert.AreSame(lookupList, solutionRenamer.LookupListsWithPropsToBeDeleted.First());
            Assert.AreSame(dmProperty, lookupList.DMProperty);
            Assert.IsFalse(lookupList.Status.IsDeleted);
            Assert.IsFalse(dmProperty.Status.IsDeleted);
            //---------------Execute Test ----------------------
            solutionRenamer.Delete();
            //---------------Test Result -----------------------
            Assert.IsTrue(lookupList.Status.IsDeleted);
            Assert.IsTrue(dmProperty.Status.IsDeleted);
        }
        [Test]
        public void Test_Delete_WhenHasTwoLookupListsWithPropsToDelete_ShouldDeleteFKProp()
        {
            //If when deleting the DMSolution the user selected to Delete the Related FKProp
            // the the Related FKProp Should Be Deleted
            //---------------Set up test pack-------------------
            IDMSolution dmSolution = GetMockDMSolution();
            var solutionRenamer = new SolutionRenamer(dmSolution);


            var dmProperty = new DMProperty();
            var lookupList = new DMLookupListBusinessObject { DMProperty = dmProperty };
            solutionRenamer.AddLookupListsWithPropsToBeDeleted(lookupList);

            var dmProperty2 = new DMProperty();
            var lookupList2 = new DMLookupListBusinessObject { DMProperty = dmProperty2 };
            solutionRenamer.AddLookupListsWithPropsToBeDeleted(lookupList2);

            //---------------Assert Precondition----------------
            Assert.IsFalse(dmProperty.Status.IsDeleted);
            Assert.IsFalse(dmProperty2.Status.IsDeleted);
            //---------------Execute Test ----------------------
            solutionRenamer.Delete();
            //---------------Test Result -----------------------
            Assert.IsTrue(dmProperty.Status.IsDeleted);
            Assert.IsTrue(dmProperty2.Status.IsDeleted);
        }

        [Test]
        public void Test_Delete_WhenDMPropNull_ShouldNotThrowError()
        {
            //If when deleting the DMSolution the user selected to Delete the Related FKProp
            // the the Related FKProp Should Be Deleted
            //---------------Set up test pack-------------------
            IDMSolution dmSolution = GetMockDMSolution();
            var solutionRenamer = new SolutionRenamer(dmSolution);

            var lookupList2 = new DMLookupListBusinessObject { DMProperty = null };
            solutionRenamer.AddLookupListsWithPropsToBeDeleted(lookupList2);

            //---------------Assert Precondition----------------
            Assert.IsNull(lookupList2.DMProperty);
            //---------------Execute Test ----------------------
            solutionRenamer.Delete();
            //---------------Test Result -----------------------
            Assert.IsTrue(true, "Should not raise Error");
        }

        [Test]
        public void Test_MarkMarkAllLookupListsPropsToBeDeleted_ShouldMoveAllTosWithFKToBeDeleted()
        {
            //---------------Set up test pack-------------------
            IDMSolution dmSolution = new DMSolution();
            var lookupList = new DMLookupListBusinessObject();
            lookupList.DMProperty = new DMProperty();
            dmSolution.DMLookupListBusinessObjects.Add(lookupList);
            var lookupList2 = new DMLookupListBusinessObject();
            lookupList.DMProperty = new DMProperty();
            dmSolution.DMLookupListBusinessObjects.Add(lookupList2);
            var solutionRenamer = new SolutionRenamer(dmSolution);

            //---------------Assert Precondition----------------
            Assert.AreEqual(0, solutionRenamer.LookupListsWithPropsToBeDeleted.Count());
            Assert.AreEqual(2, dmSolution.DMLookupListBusinessObjects.Count);
            Assert.AreEqual(2, solutionRenamer.AllLookupLists.Count());
            //---------------Execute Test ----------------------
            solutionRenamer.MarkAllLookupListsPropsToBeDeleted();
            //---------------Test Result -----------------------
            Assert.AreEqual(2, solutionRenamer.LookupListsWithPropsToBeDeleted.Count());
            Assert.AreEqual(2, dmSolution.DMLookupListBusinessObjects.Count);
            Assert.AreEqual(2, solutionRenamer.AllLookupLists.Count());
        }

        [Test]
        public void Test_Delete_ShouldMarkAllUniqueConstraintsForDelete()
        {
            //---------------Set up test pack-------------------
            var ucProp = TestUtilsDMUniqueConstraintProperty.CreateUnsavedValidDMUniqueConstraintProperty();
            var dmSolution = ucProp.UniqueConstraint.Class;
            var solutionRenamer = new SolutionRenamer(dmSolution);
            //---------------Assert Precondition----------------
            Assert.IsNotNull(dmSolution);
            Assert.IsNotNull(ucProp.Property);
            Assert.IsFalse(ucProp.Status.IsDeleted);
            Assert.IsFalse(ucProp.Property.Status.IsDeleted);
            Assert.IsFalse(ucProp.UniqueConstraint.Status.IsDeleted);
            //---------------Execute Test ----------------------
            solutionRenamer.Delete();
            //---------------Test Result -----------------------
            Assert.IsTrue(ucProp.Status.IsDeleted);
            Assert.IsTrue(ucProp.Property.Status.IsDeleted);
            Assert.IsTrue(ucProp.UniqueConstraint.Status.IsDeleted);
        }
        private static IDMSolution GetMockDMSolution()
        {
            var dmSolution = MockRepository.GenerateMock<IDMSolution>();
            dmSolution.Stub(@class => @class.RelationshipsDM).Return(new FakeBusinessObjectCollection<DMRelationship>());
            dmSolution.Stub(@class => @class.DMLookupListBusinessObjects).Return(new FakeBusinessObjectCollection<DMLookupListBusinessObject>());
            return dmSolution;
        }*/
    }
}