using FireStarterModeller.BO;
using FireStarterModeller.Logic;
using FireStarterModeller.Test.BO;
using NUnit.Framework;
using Rhino.Mocks;

namespace FireStarterModeller.Test.Logic
{
    [TestFixture]
    public class TestRelationshipNameGenerator :TestBase
    {
        // Story: Implement the Relationship Naming logic (Use Case A.2)
        //
        //2.		If the Relationship has only one Property then:
        //-	Done: If the owned property name or the related property name end with ID then remove this. These modified values will be refered to as OwnerPropertyName and RelatedPropertyName.
        //-	If the RelatedPropertyName is contained within the OwnerPropertyName then the RelatedPropertyName can be removed from the OwnerPropertyName to get the RelationshipAdjective. 
        //    This RelationshipAdjective can then be combined with the Related Class Name to get the PreliminaryRelationshipName passed through to Step 5.
        //-	Otherwise, the OwnerPropertyName should be used for the PreliminaryRelationshipName passed through to Step 5.
        //3.		If the Relationship has more than one RelatedProperty then:
        //-	All of the OwnerPropertyName values can be combined to get the PreliminaryRelationshipName passed through to Step 5.
        //4.		If the Relationship does not have any RelatedProperties then:
        //-	The Related Class Name should be passed through to Step 5 as the PreliminaryRelationshipName.
        //5.		The PreliminaryRelationshipName is modified as follows:
        //-	If the Relationship is a single relationship then the PreliminaryRelationshipName should be Singularised to get the Final Relationship Name which is returned to the Actor.
        //-	If the Relationship is a multiple relationship then the PreliminaryRelationshipName should be Pluralised to get the Final Relationship Name which is returned to the Actor

        [Test]
        public void Test_Construct()
        {
            //---------------Set up test pack-------------------
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            RelationshipNameGenerator relationshipNameGenerator = new RelationshipNameGenerator();
            //---------------Test Result -----------------------
            Assert.IsInstanceOf(typeof(IRelationshipNameGenerator), relationshipNameGenerator);
        }


        [Test]
        public void Test_CreateRelationshipName_WhenSingle_WithPrimaryKeyField_ShouldUseRelatedClassName()
        {
            //---------------Set up test pack-------------------
            IRelationshipNameGenerator relationshipNameGenerator = new RelationshipNameGenerator();
            const string expectedRelName = "MyRelationship";
            DMRelationship dmRelationship = CreateMockSingleRelationshipWithProperty(GetRandomString() + "ID");
            dmRelationship.Stub(relationship => relationship.HasAllPkProps()).Return(true);
            dmRelationship.RelatedClassName = expectedRelName;
            //---------------Assert Precondition----------------
            Assert.IsTrue(dmRelationship.HasAllPkProps());
            //---------------Execute Test ----------------------
            string relationshipName = relationshipNameGenerator.CreateRelationshipName(dmRelationship);
            //---------------Test Result -----------------------
            Assert.AreEqual(expectedRelName, relationshipName);
        }

        [Test]
        public void Test_CreateRelationshipName_WhenSingle_WithIDField_ShouldDropIDSuffix()
        {
            //---------------Set up test pack-------------------
            IRelationshipNameGenerator relationshipNameGenerator = new RelationshipNameGenerator();
            const string relationshipPropertyName = "MyRelationship";
            DMRelationship dmRelationship = CreateSingleRelationshipWithProperty(relationshipPropertyName + "ID");
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            string relationshipName = relationshipNameGenerator.CreateRelationshipName(dmRelationship);
            //---------------Test Result -----------------------
            Assert.AreEqual(relationshipPropertyName, relationshipName);
        }

        [Test]
        public void Test_CreateRelationshipName_WhenSingle_WithNoIDField_ShouldCopyPropName()
        {
            //---------------Set up test pack-------------------
            IRelationshipNameGenerator relationshipNameGenerator = new RelationshipNameGenerator();
            const string relationshipPropertyName = "MyRelationship";
            DMRelationship dmRelationship = CreateSingleRelationshipWithProperty(relationshipPropertyName);
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            string relationshipName = relationshipNameGenerator.CreateRelationshipName(dmRelationship);
            //---------------Test Result -----------------------
            Assert.AreEqual(relationshipPropertyName, relationshipName);
        }

        [Test]
        public void Test_CreateRelationshipName_WhenSingle_WithIDInTheMiddle_ShouldReturnFullPropertyName()
        {
            //---------------Set up test pack-------------------
            IRelationshipNameGenerator relationshipNameGenerator = new RelationshipNameGenerator();
            const string relationshipPropertyName = "MyRelatIDionship";
            DMRelationship dmRelationship = CreateSingleRelationshipWithProperty(relationshipPropertyName);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            string relationshipName = relationshipNameGenerator.CreateRelationshipName(dmRelationship);

            //---------------Test Result -----------------------
            Assert.AreEqual(relationshipPropertyName, relationshipName);
        }
        [Test]
        public void Test_RenameRel_WhenSingle_WithIDInTheMiddle_AndHasName_ShouldReturnFullPropertyName()
        {
            //---------------Set up test pack-------------------
            IRelationshipNameGenerator relationshipNameGenerator = new RelationshipNameGenerator();
            const string relationshipPropertyName = "MyRelatIDionship";
            DMRelationship dmRelationship = CreateSingleRelationshipWithProperty(relationshipPropertyName);
            dmRelationship.RelationshipName = GetRandomString();
            //---------------Assert Precondition----------------
            Assert.IsNotNull(dmRelationship.RelationshipName);
            //---------------Execute Test ----------------------
            string relationshipName = relationshipNameGenerator.RenameRelationship(dmRelationship);

            //---------------Test Result -----------------------
            Assert.AreEqual(relationshipPropertyName, relationshipName);
        }

        private static string GetRandomString()
        {
            return TestUtilsShared.GetRandomString();
        }

        [Test]
        public void Test_CreateRelationshipName_WhenSingle_WithNullPropertyName_ShouldReturnNull()
        {
            //---------------Set up test pack-------------------
            IRelationshipNameGenerator relationshipNameGenerator = new RelationshipNameGenerator();
            const string relationshipPropertyName = null;
            DMRelationship dmRelationship = CreateSingleRelationshipWithProperty(relationshipPropertyName);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            string relationshipName = relationshipNameGenerator.CreateRelationshipName(dmRelationship);

            //---------------Test Result -----------------------
            Assert.AreEqual(relationshipPropertyName, relationshipName);
        }

        [Test]
        public void Test_CreateRelationshipName_WhenSingle_WithNullProperty_ShouldUseRelatedClassName()
        {
            //---------------Set up test pack-------------------
            IRelationshipNameGenerator relationshipNameGenerator = new RelationshipNameGenerator();
            const string relationshipPropertyName = null;
            DMRelationship dmRelationship = CreateSingleRelationshipWithProperty(relationshipPropertyName);
            dmRelationship.RelationshipProperties[0].Property = null;
            dmRelationship.RelatedClass = TestUtilsShared.CreateUnsavedValidDMClass();
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            string relationshipName = relationshipNameGenerator.CreateRelationshipName(dmRelationship);

            //---------------Test Result -----------------------
            Assert.AreEqual(dmRelationship.RelatedClass.ClassNameBO, relationshipName);
        }

        [Test]
        public void Test_CreateRelationshipName_WhenSingle_WithNullPropertyAndNullRelatedClass_ShouldReturnNull()
        {
            //---------------Set up test pack-------------------
            IRelationshipNameGenerator relationshipNameGenerator = new RelationshipNameGenerator();
            const string relationshipPropertyName = null;
            DMRelationship dmRelationship = CreateSingleRelationshipWithProperty(relationshipPropertyName);
            dmRelationship.RelationshipProperties[0].Property = null;
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            string relationshipName = relationshipNameGenerator.CreateRelationshipName(dmRelationship);

            //---------------Test Result -----------------------
            TestUtilsShared.AssertStringEmpty(relationshipName, "relationshipName");
        }

        [Test]
        public void Test_CreateRelationshipName_WhenMultiple_ShouldReturnChildPlural()
        {
            //---------------Set up test pack-------------------
            IRelationshipNameGenerator relationshipNameGenerator = new RelationshipNameGenerator();
            const string fkPropertyName = "MyParentID";
            const string childClassName = "Property";
            DMRelationship dmRelationship = CreateMultipleRelationshipWithProperty(childClassName, fkPropertyName);
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            string relationshipName = relationshipNameGenerator.CreateRelationshipName(dmRelationship);
            //---------------Test Result -----------------------
            Assert.AreEqual("Properties", relationshipName);
        }

        private static DMRelationship CreateMockSingleRelationshipWithProperty(string relationshipPropertyName)
        {
            DMRelationship dmRelationship = MockRepository.GenerateStub<DMRelationship>();
            var relProps = new FakeBusinessObjectCollection<DMRelationshipProperty>();
            dmRelationship.Stub(relationship => relationship.RelationshipProperties).Return(relProps);
            dmRelationship.Cardinality = Cardinality.Single;
            DMRelationshipProperty dmRelationshipProperty = new DMRelationshipProperty();
            dmRelationshipProperty.Property = new DMProperty { PropertyName = relationshipPropertyName };
            dmRelationship.RelationshipProperties.Add(dmRelationshipProperty);
            return dmRelationship;
        }
        private static DMRelationship CreateSingleRelationshipWithProperty(string relationshipPropertyName)
        {
            DMRelationship dmRelationship = new DMRelationship();
            dmRelationship.Cardinality = Cardinality.Single;
            DMRelationshipProperty dmRelationshipProperty = new DMRelationshipProperty();
            dmRelationshipProperty.Property = new DMProperty { PropertyName = relationshipPropertyName };
            dmRelationship.RelationshipProperties.Add(dmRelationshipProperty);
            return dmRelationship;
        }

        private static DMRelationship CreateMultipleRelationshipWithProperty(string childClassName, string fkPropertyName)
        {
            DMRelationship dmRelationship = new DMRelationship();
            dmRelationship.Cardinality = Cardinality.Multiple;
            dmRelationship.ClassDM = TestUtilsDMClass.CreateUnsavedValidDMClass();
            DMProperty dmPKProperty = TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmRelationship.ClassDM);

            IDMRelationship reverseRelationship = dmRelationship.ReverseRelationship;
            reverseRelationship.ClassDM = TestUtilsDMClass.CreateUnsavedValidDMClass(childClassName);
            DMProperty dmFKProperty = TestUtilsDMProperty.CreateUnsavedValidDMProperty(reverseRelationship.ClassDM);
            dmFKProperty.PropertyName = fkPropertyName;
            
            DMRelationshipProperty dmRelationshipProperty = new DMRelationshipProperty();
            dmRelationshipProperty.Property = dmPKProperty;
            dmRelationshipProperty.ReverseRelationshipProperty.Property = dmFKProperty;
            dmRelationship.RelationshipProperties.Add(dmRelationshipProperty);
            return dmRelationship;
        }

    }
}