using System.Data;
using System.Data.Common;
using Habanero.Base;
using Habanero.BO;

namespace Firestarter.DB.Schema.Logic
{
    public abstract class DBSchemaExtractor : IDBSchemaExtractor
    {
        protected readonly IDbConnection _dbConnection;

        protected DBSchemaExtractor(IDbConnection idbConnection)
        {
            _dbConnection = idbConnection;
        }

        /// <summary>
        /// Gets the current Database Name from the database connection.
        /// </summary>
        public virtual string DatabaseName
        {
            get { return _dbConnection.Database; }
        }

        /// <summary>
        /// Gets the current Catalog Name from the database connection.
        /// </summary>
        public abstract string CatalogName { get; }

        /// <summary>
        /// Gets the current Schema Name from the database connection.
        /// </summary>
        public abstract string SchemaName { get; }
        
        protected void OpenConnection()
        {
            if (_dbConnection.State == ConnectionState.Closed) _dbConnection.Open();
        }

        protected virtual DataTable GetDataTable(string sql)
        {
            OpenConnection();
            DbDataAdapter adapter = CreateDataAdapter(sql);
            DataTable dataTable = new DataTable();
            adapter.Fill(dataTable);
            return dataTable;
        }

        protected abstract DbDataAdapter CreateDataAdapter(string sql);

        /// <summary>
        /// This method is the specific for the database implementation for fetching the 
        /// DataTable representing the databases for the connection with a specified database name.
        /// This database name might be used to filter the catalog, schema, or both.
        /// </summary>
        /// <param name="databaseName">The name of the database to filter by.</param>
        /// <returns>A DataTable populated with the Databases information.</returns>
        public virtual DataTable GetDatabasesDataTable(string databaseName)
        {
            return GetSchemaDataTable("Databases", databaseName);
        }

        public abstract DataTable GetDatabasesDataTable(string catalogName, string schemaName);

        public virtual DataTable GetDataTypesDataTable()
        {
            return GetSchemaDataTable("DataTypes", null, null, null);
        }

        public DataTable GetTablesDataTable()
        {
            return GetTablesDataTable(CatalogName, SchemaName);
        }
        protected virtual DataTable GetTablesDataTable(string catalogName, string schemaName)
        {
            return GetSchemaDataTable("Tables", catalogName, schemaName, null);
        }

        public DataTable GetColumnsDataTable(string tableName)
        {
            return GetColumnsDataTable(CatalogName, SchemaName, tableName);
        }
        protected virtual DataTable GetColumnsDataTable(string catalogName, string schemaName, string tableName)
        {
            return GetSchemaDataTable("Columns", catalogName, schemaName, tableName);
        }

        public DataTable GetPrimaryKeyColumnsDataTable(string tableName)
        {
            return GetPrimaryKeyColumnsDataTable(CatalogName, SchemaName, tableName);
        }
        // This needs to be overridden because there is a difference in columns
        // General:  TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,KEY_CATALOG,KEY_SCHEMA,KEY_NAME
        // MySQL:    INDEX_CATALOG,INDEX_SCHEMA,INDEX_NAME,TABLE_NAME,UNIQUE,PRIMARY
        protected virtual DataTable GetPrimaryKeyColumnsDataTable(string catalogName, string schemaName, string tableName)
        {
            //return GetSchemaDataTable("PrimaryKeys", catalogName, schemaName, tableName);
            string sql = BuildConstraintColumnsSql("PRIMARY KEY", catalogName, schemaName, tableName, false);
            return GetDataTable(sql);
        }

        public DataTable GetUniqueKeyColumnsDataTable(string tableName)
        {
            return GetUniqueKeyColumnsDataTable(CatalogName, SchemaName, tableName);
        }
        // This needs to be overridden because there is a difference in columns
        // General:  TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,KEY_CATALOG,KEY_SCHEMA,KEY_NAME
        // MySQL:    INDEX_CATALOG,INDEX_SCHEMA,INDEX_NAME,TABLE_NAME,UNIQUE,PRIMARY
        protected virtual DataTable GetUniqueKeyColumnsDataTable(string catalogName, string schemaName, string tableName)
        {
            //return GetSchemaDataTable("UniqueKeys", catalogName, schemaName, tableName);
            string sql = BuildConstraintColumnsSql("UNIQUE", catalogName, schemaName, tableName, false);
            return GetDataTable(sql);
        }

        protected virtual string BuildConstraintColumnsSql(string contraintType, string catalogName, string schemaName, string tableName, bool includeCatalog)
        {
            string sql = "Select COL_USAGE.TABLE_CATALOG , COL_USAGE.TABLE_SCHEMA, COL_USAGE.TABLE_NAME, " +
                "TAB_CONST.CONSTRAINT_CATALOG as KEY_CATALOG, TAB_CONST.CONSTRAINT_SCHEMA as KEY_SCHEMA, " +
                "TAB_CONST.CONSTRAINT_NAME as KEY_NAME, COL_USAGE.COLUMN_NAME, COL_USAGE.ORDINAL_POSITION " +
                "from INFORMATION_SCHEMA.KEY_COLUMN_USAGE COL_USAGE " +
                "inner join INFORMATION_SCHEMA.TABLE_CONSTRAINTS TAB_CONST " +
                "on COL_USAGE.CONSTRAINT_NAME = TAB_CONST.CONSTRAINT_NAME " +
                "and COL_USAGE.CONSTRAINT_SCHEMA = TAB_CONST.CONSTRAINT_SCHEMA " +
                 (includeCatalog ? "and COL_USAGE.CONSTRAINT_CATALOG = TAB_CONST.CONSTRAINT_CATALOG " : "") +
                "and COL_USAGE.TABLE_NAME = TAB_CONST.TABLE_NAME " +
                "and COL_USAGE.TABLE_SCHEMA = TAB_CONST.TABLE_SCHEMA " +
                 (includeCatalog ? "and COL_USAGE.TABLE_CATALOG = TAB_CONST.TABLE_CATALOG " : "") +
                 (includeCatalog ? GetSqlWithAnd("COL_USAGE.TABLE_CATALOG", catalogName) : "") +
                GetSqlWithAnd("COL_USAGE.TABLE_SCHEMA", schemaName) +
                GetSqlWithAnd("COL_USAGE.TABLE_NAME", tableName) +
                GetSqlWithAnd("TAB_CONST.CONSTRAINT_TYPE", contraintType);
            //SqlUtilities.AddSqlWhereParameter(ref sql, "TAB_CONST.CONSTRAINT_TYPE", contraintType);
            //if (includeCatalog && !string.IsNullOrEmpty(catalogName)) 
            //    SqlUtilities.AddSqlWhereParameter(ref sql, "TAB_CONST.CONSTRAINT_NAME", catalogName);
            //if (!string.IsNullOrEmpty(schemaName)) SqlUtilities.AddSqlWhereParameter(ref sql, "TAB_CONST.TABLE_SCHEMA", schemaName);
            //if (!string.IsNullOrEmpty(tableName)) SqlUtilities.AddSqlWhereParameter(ref sql, "TAB_CONST.TABLE_NAME", tableName);
            sql += " ORDER BY " + (includeCatalog ? "TAB_CONST.TABLE_CATALOG, " : "") +
                " TAB_CONST.TABLE_SCHEMA, TAB_CONST.TABLE_NAME, " +
                " TAB_CONST.CONSTRAINT_NAME, COL_USAGE.ORDINAL_POSITION ";
            return sql;
        }

        public string GetSqlWithAnd(string columnName, string value)
        {
            if (value == null) return null;
            return string.Format(" and {0} = '{1}' ", columnName, value);
        }

        public DataTable GetForeignKeyColumnsDataTable(string tableName, string foreignKeyName)
        {
            return GetForeignKeyColumnsDataTable(CatalogName, SchemaName, tableName, foreignKeyName);
        }
        /// <summary>
        /// Creates a <see cref="DataTable"/> for Foreign Key columns with the following <see cref="DataColumn"/>s:
        /// TABLE_CATALOG, TABLE_SCHEMA, TABLE_NAME, 
        /// KEY_CATALOG, KEY_SCHEMA, KEY_NAME, COLUMN_NAME, REFERENCED_TABLE_CATALOG, 
        /// REFERENCED_TABLE_SCHEMA, REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME, ORDINAL_POSITION
        /// </summary>
        /// <param name="catalogName">The catalog name of the database that you are reading the foreign keys for </param>
        /// <param name="schemaName">The schema name of the database that you are reading the foreign keys for </param>
        /// <param name="tableName">The table name of the particular table that you are reading the foreign keys for </param>
        /// <param name="foreignKeyName">The particular foreign key of the particular table that you are reading the foreign keys for </param>
        /// <returns></returns>
        protected virtual DataTable GetForeignKeyColumnsDataTable(string catalogName, string schemaName, string tableName, string foreignKeyName)
        {
            string sqlRelationships = BuildForeignKeyColumnsSql(catalogName, schemaName, tableName, true);
            return GetDataTable(sqlRelationships);
        }

        protected virtual string BuildForeignKeyColumnsSql(string catalogName, string schemaName, string tableName, bool includeCatalog)
        {
            string sqlRelationships = @"SELECT" +
                "  TAB_CONST.CONSTRAINT_CATALOG as KEY_CATALOG, TAB_CONST.CONSTRAINT_SCHEMA as KEY_SCHEMA, " +
                "  TAB_CONST.CONSTRAINT_NAME as KEY_NAME, " +
                (includeCatalog ? "  TAB_CONST.TABLE_CATALOG as TABLE_CATALOG, " : "") +
                "  TAB_CONST.TABLE_SCHEMA as TABLE_SCHEMA, " +
                "  TAB_CONST.TABLE_NAME as TABLE_NAME, " +
                "  FK_COLS.COLUMN_NAME as COLUMN_NAME, " +
                (includeCatalog ? "  PK.TABLE_CATALOG as REFERENCED_TABLE_CATALOG, " : "") +
                "  PK.TABLE_SCHEMA as REFERENCED_TABLE_SCHEMA, " +
                "  PK.TABLE_NAME as REFERENCED_TABLE_NAME, " +
                "  PK_COLS.COLUMN_NAME as REFERENCED_COLUMN_NAME, " +
                "  FK_COLS.ORDINAL_POSITION " +
                "FROM INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS REF_CONST " +
                "  INNER JOIN INFORMATION_SCHEMA.TABLE_CONSTRAINTS TAB_CONST " +
                "    ON " +
                (includeCatalog
                   ? "REF_CONST.CONSTRAINT_CATALOG = TAB_CONST.CONSTRAINT_CATALOG AND "
                   : "") +
                "	      REF_CONST.CONSTRAINT_SCHEMA = TAB_CONST.CONSTRAINT_SCHEMA " +
                "	  AND REF_CONST.CONSTRAINT_NAME = TAB_CONST.CONSTRAINT_NAME " +
                "	  AND TAB_CONST.CONSTRAINT_TYPE = 'FOREIGN KEY' " +
                "  INNER JOIN INFORMATION_SCHEMA.TABLE_CONSTRAINTS PK " +
                "	ON " +
                (includeCatalog
                   ? "REF_CONST.UNIQUE_CONSTRAINT_CATALOG = PK.CONSTRAINT_CATALOG AND "
                   : "") +
                "	      REF_CONST.UNIQUE_CONSTRAINT_SCHEMA = PK.CONSTRAINT_SCHEMA " +
                "	  AND REF_CONST.UNIQUE_CONSTRAINT_NAME = PK.CONSTRAINT_NAME " +
                "	  AND PK.CONSTRAINT_TYPE = 'PRIMARY KEY' " +
                //"  INNER JOIN INFORMATION_SCHEMA.KEY_COLUMN_USAGE FK_COLS " +
                //"	ON REF_CONST.CONSTRAINT_NAME = FK_COLS.CONSTRAINT_NAME " +
                //"  INNER JOIN INFORMATION_SCHEMA.KEY_COLUMN_USAGE PK_COLS " +
                //"	ON PK.CONSTRAINT_NAME = PK_COLS.CONSTRAINT_NAME ";
                "  INNER JOIN INFORMATION_SCHEMA.KEY_COLUMN_USAGE AS FK_COLS " +
                "    ON TAB_CONST.CONSTRAINT_SCHEMA = FK_COLS.CONSTRAINT_SCHEMA " +
                (includeCatalog? " AND TAB_CONST.CONSTRAINT_CATALOG = FK_COLS.CONSTRAINT_CATALOG ": "") +
                "      AND TAB_CONST.CONSTRAINT_NAME = FK_COLS.CONSTRAINT_NAME " +
                "      AND TAB_CONST.TABLE_SCHEMA = FK_COLS.TABLE_SCHEMA " +
                (includeCatalog ? " AND TAB_CONST.TABLE_CATALOG = FK_COLS.TABLE_CATALOG " : "") +
                "      AND TAB_CONST.TABLE_NAME = FK_COLS.TABLE_NAME " +
                "  INNER JOIN INFORMATION_SCHEMA.KEY_COLUMN_USAGE AS PK_COLS " +
                "    ON PK.CONSTRAINT_SCHEMA = PK_COLS.CONSTRAINT_SCHEMA " +
                (includeCatalog ? " AND PK.CONSTRAINT_CATALOG = PK_COLS.CONSTRAINT_CATALOG " : "") +
                "      AND PK.CONSTRAINT_NAME = PK_COLS.CONSTRAINT_NAME " +
                "      AND PK.TABLE_SCHEMA = PK_COLS.TABLE_SCHEMA " +
                (includeCatalog ? " AND PK.TABLE_CATALOG = PK_COLS.TABLE_CATALOG " : "") +
                "      AND PK.TABLE_NAME = PK_COLS.TABLE_NAME " +
                "      AND FK_COLS.ORDINAL_POSITION = PK_COLS.ORDINAL_POSITION ";

            //SqlUtilities.AddSqlWhereParameter(ref sqlRelationships, "TAB_CONST.CONSTRAINT_TYPE", "FOREIGN KEY");

            if (includeCatalog && !string.IsNullOrEmpty(catalogName)) 
                SqlUtilities.AddSqlWhereParameter(ref sqlRelationships, "TAB_CONST.TABLE_CATALOG", catalogName);
            if (!string.IsNullOrEmpty(schemaName)) SqlUtilities.AddSqlWhereParameter(ref sqlRelationships, "TAB_CONST.TABLE_SCHEMA", schemaName);
            if (!string.IsNullOrEmpty(tableName)) SqlUtilities.AddSqlWhereParameter(ref sqlRelationships, "TAB_CONST.TABLE_NAME", tableName);
            sqlRelationships += " ORDER BY " + (includeCatalog ? "TAB_CONST.TABLE_CATALOG, " : "") +
                " TAB_CONST.TABLE_SCHEMA, TAB_CONST.TABLE_NAME, " +
                " TAB_CONST.CONSTRAINT_NAME, FK_COLS.ORDINAL_POSITION ";
            return sqlRelationships;
        }
        //public virtual DataTable GetForeignKeyColumnsDataTable(string catalogName, string schemaName, string tableName, string foreignKeyName)
        //{
        //    return GetSchemaDataTable("ForeignKeyColumns", catalogName, schemaName, tableName, foreignKeyName);
        //}

        protected DataTable GetSchemaDataTable(string collectionName, string catalogName, string schemaName, string tableName)
        {
            return GetSchemaDataTable(collectionName, catalogName, schemaName, tableName, null);
        }

        protected DataTable GetSchemaDataTable(string collectionName, string catalogName, string schemaName, string tableName, string identifierName)
        {
            string[] parameters = CreateParameters(catalogName, schemaName, tableName, identifierName);
            return GetSchemaDataTable(collectionName, parameters);
        }

        protected virtual string[] CreateParameters(string catalogName, string schemaName, string tableName, string identifierName)
        {
            if (identifierName != null) return new[] { catalogName, schemaName, tableName, identifierName };
            if (tableName != null) return new[] { catalogName, schemaName, tableName };
            if (schemaName != null) return new[] { catalogName, schemaName };
            return catalogName != null ? new[] { catalogName } : null;
        }

        protected DataTable GetSchemaDataTable(string collectionName, params string[] parameters)
        {
            OpenConnection();
            DbConnection dbConnection = (DbConnection)_dbConnection;
            return dbConnection.GetSchema(collectionName, parameters);
        }
    }
}
