using System;
using System.Data;
using System.Data.Common;
using MySql.Data.MySqlClient;

namespace Firestarter.DB.Schema.Logic
{
    public class DBSchemaExtractorMySQL: DBSchemaExtractor
    {
        public DBSchemaExtractorMySQL(IDbConnection idbConnection) : base(idbConnection)
        {
        }

        /// <summary>
        /// Gets the current Catalog Name from the database connection.
        /// </summary>
        public override string CatalogName
        {
            get { return null; }
        }

        /// <summary>
        /// Gets the current Schema Name from the database connection.
        /// </summary>
        public override string SchemaName
        {
            get { return _dbConnection.Database.ToLowerInvariant(); }
        }

        /// <summary>
        /// This method is the specific for the database implementation for fetching the 
        /// DataTable representing the databases for the connection with a specified database name.
        /// This database name might be used to filter the catalog, schema, or both.
        /// This needs to be overridden because there is a difference in columns:
        /// General:  CATALOG_NAME, SCHEMA_NAME
        /// MYSQL:    CATALOG_NAME,database_name,DEFAULT_CHARACTER_SET_NAME,DEFAULT_COLLATION_NAME,SQL_PATH
        /// </summary>
        /// <param name="databaseName">The name of the database to filter by.</param>
        /// <returns>A DataTable populated with the Databases information.</returns>
        public override DataTable GetDatabasesDataTable(string databaseName)
        {
            DataTable dataTable = base.GetDatabasesDataTable(databaseName);
            dataTable.Columns["database_name"].ColumnName = DBSchemaTemplate.Naming.SchemaName;
            return dataTable;
        }

        public override DataTable GetDatabasesDataTable(string catalogName, string schemaName)
        {
            return GetDatabasesDataTable(schemaName);
        }

        // This needs to be overridden because there is a difference in one column
        // The IsLiteralSupported column is named IsLiteralsSupported in MySQL
        public override DataTable GetDataTypesDataTable()
        {
            DataTable dataTable = base.GetDataTypesDataTable();
            dataTable.Columns["IsLiteralsSupported"].ColumnName = DBSchemaTemplate.Naming.IsLiteralSupported;
            return dataTable;
        }

        //General:  TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,COLUMN_NAME,ORDINAL_POSITION,COLUMN_DEFAULT,IS_NULLABLE,DATA_TYPE,CHARACTER_MAXIMUM_LENGTH,NUMERIC_PRECISION,NUMERIC_SCALE, IS_AUTO_INCREMENTING
        //MySQL:    TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,COLUMN_NAME,ORDINAL_POSITION,COLUMN_DEFAULT,IS_NULLABLE,DATA_TYPE,CHARACTER_MAXIMUM_LENGTH,NUMERIC_PRECISION,NUMERIC_SCALE,CHARACTER_SET_NAME,COLLATION_NAME,COLUMN_TYPE,COLUMN_KEY,EXTRA,PRIVILEGES,COLUMN_COMMENT
        protected override DataTable GetColumnsDataTable(string catalogName, string schemaName, string tableName)
        {
            DataTable dataTable = base.GetColumnsDataTable(catalogName, schemaName, tableName);
            DataTableUtilities.AddColumn(dataTable, DBSchemaTemplate.Naming.ColumnIsAutoIncrement, "EXTRA = 'auto_increment'");
            return dataTable;
        }



        /// <summary>
        /// Creates a <see cref="DataTable"/> for Foreign Key columns with the following <see cref="DataColumn"/>s:
        /// TABLE_CATALOG, TABLE_SCHEMA, TABLE_NAME, 
        /// KEY_CATALOG, KEY_SCHEMA, KEY_NAME, COLUMN_NAME, REFERENCED_TABLE_CATALOG, 
        /// REFERENCED_TABLE_SCHEMA, REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME, ORDINAL_POSITION
        /// </summary>
        /// <param name="catalogName">The catalog name of the database that you are reading the foreign keys for </param>
        /// <param name="schemaName">The schema name of the database that you are reading the foreign keys for </param>
        /// <param name="tableName">The table name of the particular table that you are reading the foreign keys for </param>
        /// <param name="foreignKeyName">The particular foreign key of the particular table that you are reading the foreign keys for </param>
        /// <returns></returns>
        protected override DataTable GetForeignKeyColumnsDataTable(string catalogName, string schemaName, string tableName, string foreignKeyName)
        {
            string sqlRelationships = BuildForeignKeyColumnsSql(catalogName, schemaName, tableName, false);
            return GetDataTable(sqlRelationships);
        }

        protected override string BuildForeignKeyColumnsSql(string catalogName, string schemaName, string tableName, bool includeCatalog)
        {
            string sqlRelationships =
                "select COL_USAGE.TABLE_CATALOG , COL_USAGE.TABLE_SCHEMA, COL_USAGE.TABLE_NAME, "
                + "TAB_CONST.CONSTRAINT_CATALOG as KEY_CATALOG, TAB_CONST.CONSTRAINT_SCHEMA as KEY_SCHEMA, TAB_CONST.CONSTRAINT_NAME as KEY_NAME, COL_USAGE.COLUMN_NAME, "
                + "COL_USAGE.TABLE_CATALOG as REFERENCED_TABLE_CATALOG, REFERENCED_TABLE_SCHEMA, "
                + "REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME, COL_USAGE.ORDINAL_POSITION "
                + "from INFORMATION_SCHEMA.KEY_COLUMN_USAGE COL_USAGE "
                + "left join INFORMATION_SCHEMA.TABLE_CONSTRAINTS TAB_CONST "
                + "on COL_USAGE.CONSTRAINT_NAME = TAB_CONST.CONSTRAINT_NAME "
                + "and COL_USAGE.TABLE_SCHEMA = TAB_CONST.TABLE_SCHEMA "
                + "and COL_USAGE.TABLE_NAME = TAB_CONST.TABLE_NAME " +
                 (includeCatalog ? "and COL_USAGE.TABLE_CATALOG = TAB_CONST.TABLE_CATALOG " : "");

            SqlUtilities.AddSqlWhereParameter(ref sqlRelationships, "TAB_CONST.CONSTRAINT_TYPE", "FOREIGN KEY");

            if (includeCatalog && !string.IsNullOrEmpty(catalogName))
                SqlUtilities.AddSqlWhereParameter(ref sqlRelationships, "TAB_CONST.CONSTRAINT_NAME", catalogName);
            if (!string.IsNullOrEmpty(tableName)) SqlUtilities.AddSqlWhereParameter(ref sqlRelationships, "TAB_CONST.TABLE_NAME", tableName);
            if (!string.IsNullOrEmpty(schemaName)) SqlUtilities.AddSqlWhereParameter(ref sqlRelationships, "TAB_CONST.TABLE_SCHEMA", schemaName);
            sqlRelationships += " ORDER BY " + (includeCatalog ? "TAB_CONST.TABLE_CATALOG, " : "") +
                " TAB_CONST.TABLE_SCHEMA, TAB_CONST.TABLE_NAME, " +
                " TAB_CONST.CONSTRAINT_NAME, COL_USAGE.ORDINAL_POSITION ";
            return sqlRelationships;
        }

        protected override DbDataAdapter CreateDataAdapter(string sql)
        {
            return new MySqlDataAdapter(sql, (MySqlConnection) _dbConnection);
        }

        //private static void FixIndexColumns(DataTable dataTable, string keyNoun)
        //{
        //    if (dataTable == null) return;
        //    dataTable.Columns[keyNoun + "_CATALOG"].ColumnName = DBSchemaTemplate.Naming.KeyCatalog;
        //    dataTable.Columns[keyNoun + "_SCHEMA"].ColumnName = DBSchemaTemplate.Naming.KeySchema;
        //    dataTable.Columns[keyNoun + "_NAME"].ColumnName = DBSchemaTemplate.Naming.KeyName;

        //    DataTableUtilities.AddColumn(dataTable, DBSchemaTemplate.Naming.TableCatalog, DBSchemaTemplate.Naming.KeyCatalog);
        //    DataTableUtilities.AddColumn(dataTable, DBSchemaTemplate.Naming.TableSchema, DBSchemaTemplate.Naming.KeySchema);
        //}

        //        // This needs to be overridden because there is a difference in columns
        //        // General:  TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,KEY_CATALOG,KEY_SCHEMA,KEY_NAME,COLUMN_NAME,ORDINAL_POSITION
        //        // MySQL:    INDEX_CATALOG,INDEX_SCHEMA,INDEX_NAME,TABLE_NAME,COLUMN_NAME,ORDINAL_POSITION
        //        public override DataTable GetKeyColumnsDataTable(string catalogName, string schemaName, string tableName, string keyName)
        //        {
        //            DataTable dataTable = base.GetKeyColumnsDataTable(catalogName, schemaName, tableName, null);
        //            FixIndexColumns(dataTable, "INDEX");
        //            return dataTable;
        //        }

        //        // This needs to be overridden because there is a difference in columns
        //        // General:  TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,KEY_CATALOG,KEY_SCHEMA,KEY_NAME, REFERENCED_TABLE_CATALOG,REFERENCED_TABLE_SCHEMA,REFERENCED_TABLE_NAME
        //        // MYSQL:    CONSTRAINT_CATALOG,CONSTRAINT_SCHEMA,CONSTRAINT_NAME,TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,REFERENCED_TABLE_CATALOG,REFERENCED_TABLE_SCHEMA,REFERENCED_TABLE_NAME
        //        public override DataTable GetForeignKeysDataTable(string catalogName, string schemaName, string tableName)
        //        {
        //            DataTable dataTable = base.GetForeignKeysDataTable(catalogName, schemaName, tableName);
        //            FixIndexColumns(dataTable, "CONSTRAINT");
        //            return dataTable;
        //        }
        //
        //        protected override string GetForeignKeysCollectionName()
        //        {
        //            return "Foreign Keys";
        //        }

        //        public override DataTable GetPrimaryKeyColumnDataTable(string catalogName, string schemaName, string tableName)
        //        {
        //            DataTable dataTable = base.GetPrimaryKeyColumnDataTable(catalogName, schemaName, tableName);
        ////            FixIndexColumns(dataTable, "INDEX");
        ////            DataView dataView = dataTable.DefaultView;
        ////            dataView.RowFilter = "UNIQUE = true AND PRIMARY = false";
        ////            dataTable = dataView.ToTable();
        //            return dataTable;
        //        }
    }
}
