using System.Data;
using FireStarter.Base;
using FireStarter.Base.BusinessObjectMerger;
using Firestarter.DB.Schema.BO;
using FireStarterModeller.BO;
using Habanero.Base;
using Habanero.BO;
using Habanero.DB;

namespace Firestarter.DB.Schema.Logic
{
    public class SchemaMerger
    {
        private readonly IReverseEngineering _reverseEngineering;


        public SchemaMerger(IReverseEngineering reverseEngineering)
        {
            _reverseEngineering = reverseEngineering;
        }

        public IReverseEngineering ReverseEngineering
        {
            get { return _reverseEngineering; }
        }

        public virtual void RunMerge()
        {
            IDbConnection connection = CreateOpenConnection();
            string vendor = _reverseEngineering.DatabaseConnectionInfo.Vendor.ToUpper();
            //DBSchemaExtractor schemaExtractor = new DBSchemaExtractorMySQL(connection);
            DBSchemaExtractor schemaExtractor = GetDBSchemaExtractor(vendor, connection);
            DatabaseReverseEngineer databaseReverseEngineer = GetDatabaseReverseEngineer(schemaExtractor);
            DBDatabase dbDatabase = databaseReverseEngineer.CreateDatabaseInfo(connection.Database);
            databaseReverseEngineer.PopulateDatabaseInfo(dbDatabase);
            
            IDBPostProcessor dbPostProcessor = GetPostProcessor(vendor);
            dbPostProcessor.Process(dbDatabase);

            IDBDatabase storedDBDatabase = _reverseEngineering.StoredDatabase;
            if (storedDBDatabase == null)
            {
                storedDBDatabase = new DBDatabase
                                       {
                                           Catalog = dbDatabase.Catalog,
                                           Schema = dbDatabase.Schema,
                                           DatabaseName = dbDatabase.DatabaseName,
                                           PendingChangeType = ChangeType.Add
                                       };
            }
            BusinessObjectMerger businessObjectMerger = new BusinessObjectMerger(storedDBDatabase, prop => prop.PropertyName == "Approved");
            businessObjectMerger.Import(dbDatabase);
           _reverseEngineering.StoredDatabase = storedDBDatabase;
        }

        private IDBPostProcessor GetPostProcessor(string vendor)
        {
            switch (vendor.ToUpper())
            {
                case "MYSQL":
                    return new DBMySqlPostProcessor();
                default:
                    return new DBDefaultPostProcessor();
            }
        }

        internal DatabaseReverseEngineer GetDatabaseReverseEngineer(DBSchemaExtractor schemaExtractor)
        {
            DatabaseReverseEngineer databaseReverseEngineer = new DatabaseReverseEngineer(schemaExtractor);
            string vendorName = _reverseEngineering.DatabaseConnectionInfo.Vendor.ToUpper();
            IDBDataTypeMapping dataTypeMapping = GetDBDataTypeMapping(vendorName);
            databaseReverseEngineer.DBTypeConverter = new DBTypeConverter(dataTypeMapping.GetDefaultTypeMappings());
            return databaseReverseEngineer;
        }

        private IDbConnection CreateOpenConnection()
        {
            DatabaseConfig databaseConfig = ReverseEngineering.DatabaseConnectionInfo.CreateDatabaseConfig();
            IDatabaseConnection databaseConnection = databaseConfig.GetDatabaseConnection();
            IDbConnection connection = databaseConnection.GetConnection();
            if (connection.State == ConnectionState.Closed) connection.Open();
            return connection;
        }

        internal virtual IDBDataTypeMapping GetDBDataTypeMapping(string vendorName)
        {
            switch (vendorName.ToUpper())
            {
                case "MYSQL":
                    return new DBDataTypeMappingMySQL();
                case "SQLSERVER":
                    return new DBDataTypeMappingSQLServer();
                case "ACCESS":
                    return new DBDataTypeMappingAccess();
                default:
                    return null;
            }
        }

        internal virtual DBSchemaExtractor GetDBSchemaExtractor(string vendorName, IDbConnection dbConnection)
        {

            switch (vendorName.ToUpper())
            {
                case "MYSQL":
                    return new DBSchemaExtractorMySQL(dbConnection);
                case "SQLSERVER":
                    return new DBSchemaExtractorSQLServer(dbConnection);
                case "ACCESS":
                    return new DBSchemaExtractor_Access(dbConnection);
                default:
                    return null;
            }
        }
    }
}