using System.Data;
using FireStarter.Base;
using Firestarter.DB.Schema.BO;
using Firestarter.DB.Schema.Logic;
using FireStarterModeller.BO;
using Habanero.Base;
using Habanero.BO;
using Habanero.DB;
using Habanero.Util;
using NUnit.Framework;

namespace Firestarter.DB.Schema.Test.Logic
{
    [TestFixture]
    public class TestSchemaMerger
    {
        private const string _testDatabaseName = "SchemaInfo_Test";

        [TestFixtureSetUp]
        public void SetupFixture()
        {
            TestUtils.LoadClassDefs();
            BORegistry.DataAccessor = new DataAccessorInMemory();

        }

        [SetUp]
        public void SetupTest()
        {
            TestUtils.CreateTestDatabase(_testDatabaseName);
            new TestUtilsDB_MySql().CreateSchemaTestPack();
        }

        public DatabaseConnectionInfo CreateTestDatabaseConnectionInfo()
        {
            DatabaseConfig databaseConfig = new DatabaseConfig
                ("Mysql", "localhost", _testDatabaseName, "root", "root", null);
            return DatabaseConnectionInfo.CreateFromDatabaseConfig(databaseConfig);
        }

        public DatabaseConnectionInfo CreateTestDatabaseConnectionInfo_SQLServer(out IDbConnection dbConnection)
        {
            DatabaseConfig databaseConfig = new DatabaseConfig
                ("SqlServer", "localhost", _testDatabaseName, "sa", "sa", null);
            dbConnection = databaseConfig.GetConnection();
            return DatabaseConnectionInfo.CreateFromDatabaseConfig(databaseConfig);
        }

        [Test]
        public void Test_Construct()
        {
            //---------------Set up test pack-------------------
            ReverseEngineering reverseEngineering = new ReverseEngineering();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            SchemaMerger schemaMerger = new SchemaMerger(reverseEngineering);
            //---------------Test Result -----------------------
            Assert.AreSame(reverseEngineering, schemaMerger.ReverseEngineering);
        }

        [Test]
        public void Test_ReverseEngineering_SetStoredDB()
        {
            //---------------Set up test pack-------------------
            ReverseEngineering reverseEngineering = new ReverseEngineering();
            DBDatabase database = new DBDatabase();
            //-------------Assert Preconditions -------------
            SingleRelationship<DBDatabase> storedDBRelationship =
                (SingleRelationship<DBDatabase>)reverseEngineering.Relationships["StoredDatabase"];
            Assert.IsTrue(storedDBRelationship.OwningBOHasForeignKey);
            Assert.IsNull(reverseEngineering.StoredDatabase);
            Assert.IsNull(reverseEngineering.StoredDatabaseID);
            Assert.IsNull(reverseEngineering.StoredDatabase);
            Assert.IsNull(database.StoredByReverseEngineering);
            //---------------Execute Test ----------------------
            reverseEngineering.StoredDatabase = database;
            //---------------Test Result -----------------------
            IDBDatabase storedDBDatabase = reverseEngineering.StoredDatabase;
            Assert.IsNotNull(storedDBDatabase);
            Assert.AreSame(database, storedDBDatabase);
        }

        [Test]
        public void Test_ReverseEngineering_SetStoredDBID()
        {
            //---------------Set up test pack-------------------
            ReverseEngineering reverseEngineering = new ReverseEngineering();
            DBDatabase database = new DBDatabase();
            //-------------Assert Preconditions -------------
            Assert.IsNull(reverseEngineering.StoredDatabase);
            //---------------Execute Test ----------------------
            reverseEngineering.StoredDatabaseID = database.DatabaseID;
            //---------------Test Result -----------------------
            IDBDatabase storedDBDatabase = reverseEngineering.StoredDatabase;
            Assert.IsNotNull(storedDBDatabase);
            Assert.AreSame(database, storedDBDatabase);
        }

        [Test]
        public void Test_RunMerge_FirstTime()
        {
            //---------------Set up test pack-------------------
            ReverseEngineering reverseEngineering = new ReverseEngineering();
            SchemaMerger schemaMerger = new SchemaMerger(reverseEngineering);
            reverseEngineering.DatabaseConnectionInfo = CreateTestDatabaseConnectionInfo();
            //-------------Assert Preconditions -------------
            Assert.IsNull(reverseEngineering.StoredDatabase);
            //---------------Execute Test ----------------------
            schemaMerger.RunMerge();
            //---------------Test Result -----------------------
            DBDatabase storedDBDatabase = (DBDatabase) reverseEngineering.StoredDatabase;
            Assert.IsNotNull(storedDBDatabase);
            Assert.AreEqual(7, storedDBDatabase.Tables.Count, "Incorrect number of tables");
        }

        [Test]
        public void Test_RunMerge_UpdateModel()
        {
            //---------------Set up test pack-------------------
            ReverseEngineering reverseEngineering = new ReverseEngineering();
            SchemaMerger schemaMerger = new SchemaMerger(reverseEngineering);
            reverseEngineering.DatabaseConnectionInfo = CreateTestDatabaseConnectionInfo();
            schemaMerger.RunMerge();
            DBDatabase storedDBDatabase = reverseEngineering.StoredDatabase as DBDatabase;
            const string childTableName = "childtable";
            DBTable childDBTable = storedDBDatabase.GetTableInfo(childTableName);
            storedDBDatabase.Tables.Remove(childDBTable);
            //-------------Assert Preconditions -------------
            Assert.AreEqual(6, storedDBDatabase.Tables.Count, "Incorrect number of tables");
            Assert.IsNull(storedDBDatabase.GetTableInfo(childTableName));
            //---------------Execute Test ----------------------
            schemaMerger.RunMerge();
            //---------------Test Result -----------------------
            Assert.AreEqual(7, storedDBDatabase.Tables.Count, "Incorrect number of tables");
            childDBTable = storedDBDatabase.GetTableInfo(childTableName);
            Assert.IsNotNull(childDBTable);
            Assert.AreEqual(ChangeType.Add, childDBTable.PendingChangeType);
        }


        [Test]
        public void Test_RunMerge_FirstTime_SQLServer()
        {
            //---------------Set up test pack-------------------
            TestUtilsDB_SqlServer testUtilsDbSqlServer = new TestUtilsDB_SqlServer();
            IDbConnection dbConnection;
            DatabaseConnectionInfo connectionInfo = CreateTestDatabaseConnectionInfo_SQLServer(out dbConnection);
            testUtilsDbSqlServer.ClearDatabase(dbConnection);
            testUtilsDbSqlServer.CreateSchemaTestPack();
            ReverseEngineering reverseEngineering = new ReverseEngineering();
            SchemaMerger schemaMerger = new SchemaMerger(reverseEngineering);

            try
            {
                reverseEngineering.DatabaseConnectionInfo = connectionInfo;
                //-------------Assert Preconditions -------------
                Assert.IsNull(reverseEngineering.StoredDatabase);
                //---------------Execute Test ----------------------
                schemaMerger.RunMerge();
                //---------------Test Result -----------------------

                IDBDatabase storedDBDatabase = reverseEngineering.StoredDatabase;
                Assert.IsNotNull(storedDBDatabase);
                Assert.AreEqual(7, storedDBDatabase.Tables.Count, "Incorrect number of tables");
            }
            finally
            {
                testUtilsDbSqlServer.ClearDatabase(dbConnection);
            }

        }


        //[Test]
        //public void Test_RunMerge_UpdateModel_SQLServer()
        //{
        //    //---------------Set up test pack-------------------
        //    TestUtilsDB_SqlServer testUtilsDbSqlServer = new TestUtilsDB_SqlServer();
        //    testUtilsDbSqlServer.CreateSchemaTestPack();
        //    ReverseEngineering reverseEngineering = new ReverseEngineering();
        //    SchemaMerger schemaMerger = new SchemaMerger(reverseEngineering);
        //    IDbConnection dbConnection;
        //    reverseEngineering.DatabaseConnectionInfo = CreateTestDatabaseConnectionInfo_SQLServer(out dbConnection);
        //    try
        //    {
        //        schemaMerger.RunMerge();
        //        DBDatabase storedDBDatabase = reverseEngineering.StoredDatabase;
        //        const string childTableName = "childtable";
        //        DBTable childDBTable = storedDBDatabase.GetTableInfo(childTableName);
        //        storedDBDatabase.Tables.Remove(childDBTable);
        //        //-------------Assert Preconditions -------------
        //        Assert.AreEqual(6, storedDBDatabase.Tables.Count, "Incorrect number of tables");
        //        Assert.IsNull(storedDBDatabase.GetTableInfo(childTableName));
        //        //---------------Execute Test ----------------------
        //        schemaMerger.RunMerge();
        //        //---------------Test Result -----------------------
        //        Assert.AreEqual(7, storedDBDatabase.Tables.Count, "Incorrect number of tables");
        //        childDBTable = storedDBDatabase.GetTableInfo(childTableName);
        //        Assert.IsNotNull(childDBTable);
        //        Assert.AreEqual(ChangeType.Add, childDBTable.PendingChangeType);
        //    }
        //    finally``
        //    {
        //        testUtilsDbSqlServer.ClearDatabase(dbConnection);
        //    }
        //}

        [Test]
        public void Test_GetDBDataTypeMapping_GetsMySQL()
        {
            //---------------Set up test pack-------------------
            SchemaMerger schemaMerger = new SchemaMerger(null);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            IDBDataTypeMapping mapping = schemaMerger.GetDBDataTypeMapping("mysql");
            //---------------Test Result -----------------------
            Assert.IsNotNull(mapping);
            Assert.IsInstanceOf(typeof(DBDataTypeMappingMySQL), mapping);
        }

        [Test]
        public void Test_GetDBDataTypeMapping_GetsSqlServer()
        {
            //---------------Set up test pack-------------------
            SchemaMerger schemaMerger = new SchemaMerger(null);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            IDBDataTypeMapping mapping = schemaMerger.GetDBDataTypeMapping("sqlserver");
            //---------------Test Result -----------------------
            Assert.IsNotNull(mapping);
            Assert.IsInstanceOf(typeof(DBDataTypeMappingSQLServer), mapping);
        }

        [Test]
        public void Test_GetDBDataTypeMapping_GetsAccess()
        {
            //---------------Set up test pack-------------------
            SchemaMerger schemaMerger = new SchemaMerger(null);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            IDBDataTypeMapping mapping = schemaMerger.GetDBDataTypeMapping("access");
            //---------------Test Result -----------------------
            Assert.IsNotNull(mapping);
            Assert.IsInstanceOf(typeof(DBDataTypeMappingAccess), mapping);
        }

        [Test]
        public void Test_GetDBDataTypeMapping_GetsNullIfVendorNotRecognised()
        {
            //---------------Set up test pack-------------------
            SchemaMerger schemaMerger = new SchemaMerger(null);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            IDBDataTypeMapping mapping = schemaMerger.GetDBDataTypeMapping(TestUtils.GetRandomString());
            //---------------Test Result -----------------------
            Assert.IsNull(mapping);
        }

        [Test]
        public void Test_RunMerge_SetsDBDataTypeConverter()
        {
            //---------------Set up test pack-------------------
            ReverseEngineering reverseEngineering = new ReverseEngineering();
            DatabaseConnectionInfo databaseConnectionInfo = new DatabaseConnectionInfo();
            databaseConnectionInfo.Vendor = "mysql";
            reverseEngineering.DatabaseConnectionInfo = databaseConnectionInfo;
            SchemaMergerStub schemaMerger = new SchemaMergerStub(reverseEngineering);
            //-------------Assert Preconditions -------------
            Assert.IsInstanceOf(typeof(SchemaMerger), schemaMerger);
            Assert.IsFalse(schemaMerger.MethodHasBeenHit);
            //---------------Execute Test ----------------------
            DatabaseReverseEngineer databaseReverseEngineer = schemaMerger.GetDatabaseReverseEngineer(null);
            //---------------Test Result -----------------------
            Assert.IsNotNull(databaseReverseEngineer);
            Assert.IsNotNull(databaseReverseEngineer.DBTypeConverter);
            Assert.IsNotNull(databaseReverseEngineer.DBTypeConverter.TypeMappings);
            Assert.IsTrue(schemaMerger.MethodHasBeenHit);
        }

        [Test]
        public void Test_GetDBSchemaExtractor_GetsMySQL()
        {
            //---------------Set up test pack-------------------
            IDbConnection connection = CreateOpenConnection();
            SchemaMerger schemaMerger = new SchemaMerger(null);

            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            string errorMessage;
            IDBSchemaExtractor mapping = schemaMerger.GetDBSchemaExtractor("mysql", connection);
            //---------------Test Result -----------------------
            Assert.IsNotNull(mapping);
            Assert.IsInstanceOf(typeof(DBSchemaExtractorMySQL), mapping);
        }

        [Test]
        public void Test_GetDBSchemaExtractor_GetsSqlServer()
        {
            //---------------Set up test pack-------------------
            IDbConnection connection = CreateOpenConnection();
            SchemaMerger schemaMerger = new SchemaMerger(null);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            IDBSchemaExtractor mapping = schemaMerger.GetDBSchemaExtractor("sqlserver", connection);
            //---------------Test Result -----------------------
            Assert.IsNotNull(mapping);
            Assert.IsInstanceOf(typeof(DBSchemaExtractorSQLServer), mapping);
        }

        [Test]
        public void Test_GetDBSchemaExtractor_GetsAccess()
        {
            //---------------Set up test pack-------------------
            IDbConnection connection = CreateOpenConnection();
            SchemaMerger schemaMerger = new SchemaMerger(null);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            IDBSchemaExtractor mapping = schemaMerger.GetDBSchemaExtractor("access", connection);
            //---------------Test Result -----------------------
            Assert.IsNotNull(mapping);
            Assert.IsInstanceOf(typeof(DBSchemaExtractor_Access), mapping);
        }

        [Test]
        public void Test_GetDBSchemaExtractor_GetsNullIfVendorNotRecognised()
        {
            //---------------Set up test pack-------------------
            IDbConnection connection = CreateOpenConnection();
            SchemaMerger schemaMerger = new SchemaMerger(null);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            IDBSchemaExtractor mapping = schemaMerger.GetDBSchemaExtractor(TestUtils.GetRandomString(), connection);
            //---------------Test Result -----------------------
            Assert.IsNull(mapping);
        }

        public class SchemaMergerStub : SchemaMerger
        {
            public bool MethodHasBeenHit { get; set; }
            public SchemaMergerStub(ReverseEngineering reverseEngineering)
                : base(reverseEngineering)
            {
                MethodHasBeenHit = false;
            }

            internal override IDBDataTypeMapping GetDBDataTypeMapping(string vendorName)
            {
                MethodHasBeenHit = true;
                return base.GetDBDataTypeMapping(vendorName);
            }

            internal override DBSchemaExtractor GetDBSchemaExtractor(string vendorName, IDbConnection dbConnection)
            {
                MethodHasBeenHit = true;
                return base.GetDBSchemaExtractor(vendorName, dbConnection);
            }
        }

        private IDbConnection CreateOpenConnection()
        {
            DatabaseConfig databaseConfig = CreateTestDatabaseConnectionInfo().CreateDatabaseConfig();
            IDatabaseConnection databaseConnection = databaseConfig.GetDatabaseConnection();
            IDbConnection connection = databaseConnection.GetConnection();
            if (connection.State == ConnectionState.Closed) connection.Open();
            return connection;
        }
    }
}