using System;
using System.Collections.Generic;
using System.Linq;
using FireStarter.Base;
using Firestarter.DB.Schema.BO;
using FireStarterModeller.BO;
using FireStarterModeller.Logic;
using Habanero.Base;
using Habanero.BO;
using Habanero.Util;

namespace Firestarter.DB_DomainModel.Logic
{
    public interface IDBChangesImporter
    {
        IList<IBOError> ImportDBChanges(IDBDatabase database, DMSolution solution);
    }
    internal static class IListExtentionMethod
    {
        public static IList<T> ForEach<T>(this IList<T> list, Action<T> method)
        {
            foreach (var item in list)
            {
                method(item);
            }
            return list;
        }
    }
    public class DBChangesImporter : IDBChangesImporter
    {
        private readonly IDBToDMImporterFactory _dbToDMImporterFactory;
        private readonly IDBToDMClassImporter _dbToDMClassImporter;
        private readonly IDBToDMPropertyImporter _dbToDMPropertyImporter;
        private readonly IDBToDMObjectIdentityImporter _dbToDMObjectIdentityImporter;
        private readonly IDBToDMUniqueConstraintImporter _dbToDMUniqueConstraintImporter;
        private readonly IDBToDMRelationshipImporter _dbToDMRelationshipImporter;

        public DBChangesImporter(IDBToDMImporterFactory dbToDMImporterFactory)
        {
            _dbToDMImporterFactory = dbToDMImporterFactory;
            _dbToDMClassImporter = _dbToDMImporterFactory.CreateDBToDMClassImporter();
            _dbToDMPropertyImporter = _dbToDMImporterFactory.CreateDBToDMPropertyImporter();
            _dbToDMObjectIdentityImporter = _dbToDMImporterFactory.CreateDBToDMObjectIdentityImporter();
            _dbToDMUniqueConstraintImporter = _dbToDMImporterFactory.CreateDBToDMUniqueConstraintImporter();
            _dbToDMRelationshipImporter = _dbToDMImporterFactory.CreateDBToDMRelationshipImporter();
        }

        public IList<IBOError> ImportDBChanges(IDBDatabase database, DMSolution solution)
        {
            List<IBOError> errors = new List<IBOError>();
            database.Tables.ForEach(table =>
            {
                ImportDBTableChanges(table, solution);
                if (table.PendingChangeType == ChangeType.Add && !table.PendingChangeApproved) return;
                //if (table.GetMappedDMClass() == null) return;
                table.Columns.ForEach(column => ImportDBColumnChanges(column, errors));
                ImportDBPrimaryKeyChanges(table, errors);

                table.Keys.ForEach(key => ImportDBKeyChanges(key, errors));
            });

            //foreign keys must be imported after all tables because they are dependant on multiple tables
            database.Tables.ForEach(table => table.ForeignKeys.ForEach(key => ImportDBForeignKeyChanges(key, errors)));

            database.Tables.ForEach(table =>
            {
                if (table.PendingChangeApproved) table.PendingChangeType = ChangeType.None;
            });

            if (database.PendingChangeApproved) database.PendingChangeType = ChangeType.None;

            DMSolutionPostLoadProcessor processor = new DMSolutionPostLoadProcessor(new AutoResolvingUnityContainer());
            processor.ProcessSolution(solution);
            //Fix GenerateCode -> to true for all creatd Rels.
            processor.CreatedRelationships.ForEach(rel => rel.GenerateCode = true);
            return errors;
        }

        private static void AddErrorMessages(IBusinessObject businessObject, IEnumerable<string> errorMessages, List<IBOError> errors)
        {
            errors.AddRange(errorMessages.Select(errorMessage => new BOError(errorMessage, ErrorLevel.Error) {BusinessObject = businessObject}).Cast<IBOError>());
        }

        private void ImportDBTableChanges(DBTable table, DMSolution dmSolution)
        {
            if (!table.PendingChangeApproved) return;
            switch (table.PendingChangeType)
            {
                case ChangeType.Delete:
                    table.GetMappedDMClass().MarkForDelete();
                    table.MarkForDelete();
                    return;
                case ChangeType.Add: 
                case ChangeType.Update:
                    _dbToDMClassImporter.Import(dmSolution, table);
                    break;
            }
        }

        private void ImportDBColumnChanges(DBColumn column, List<IBOError> errors)
        {
            if (!column.PendingChangeApproved) return;
            switch (column.PendingChangeType)
            {
                case ChangeType.Delete:
                    column.GetMappedDMProperty().MarkForDelete();
                    column.MarkForDelete();
                    break;
                case ChangeType.Add: 
                case ChangeType.Update:
                    if (column.Table == null) return;

                    DMClass mappedDMClass = column.Table.GetMappedDMClass();
                    if (mappedDMClass == null)
                    {
                        BOError error = new BOError(string.Format(
                                                        "Cannot add column '{0}' to class because the table '{1}' has no matching class in the domain model.",
                                                        column.ColumnName, column.Table.TableNameDB), ErrorLevel.Warning);
                        error.BusinessObject = column;
                        errors.Add(error);
                        return;
                    }
                    _dbToDMPropertyImporter.Import(mappedDMClass, column);
                    column.PendingChangeType = ChangeType.None;
                    break;
            }
        }

        private void ImportDBPrimaryKeyChanges(DBTable table, List<IBOError> errors)
        {
            DBPrimaryKey primaryKey = table.PrimaryKeyDB;
            if (primaryKey != null && primaryKey.PendingChangeApproved)
            {
                DMClass mappedDMClass = table.GetMappedDMClass();
                switch (primaryKey.PendingChangeType)
                {
                    case ChangeType.Delete:
                        mappedDMClass.GetObjectID().MarkForDelete();
                        mappedDMClass.ObjectIdentity = null;
                        table.PrimaryKeyDB = null;
                        primaryKey.MarkForDelete();
                        break;
                    case ChangeType.Add:
                    case ChangeType.Update:
                        IList<string> errorMessages;
                        _dbToDMObjectIdentityImporter.Import(mappedDMClass, primaryKey, out errorMessages);
                        AddErrorMessages(primaryKey, errorMessages, errors);
                        if (errorMessages.Count == 0)
                        {
                            primaryKey.PendingChangeType = ChangeType.None;
                            primaryKey.PrimaryKeyColumns.ForEach(column => column.PendingChangeType = ChangeType.None);
                        }
                        break;
                }
            }
        }

        private void ImportDBKeyChanges(DBKey key, List<IBOError> errors)
        {
            if (!key.PendingChangeApproved) return;
            switch (key.PendingChangeType)
            {
                case ChangeType.Delete:
                    key.GetMappedDMUniqueConstraint().MarkForDelete();
                    key.MarkForDelete();
                    break;
                case ChangeType.Add:
                case ChangeType.Update:
                    IList<string> errorMessages;
                    _dbToDMUniqueConstraintImporter.Import(key.Table.GetMappedDMClass(), key, out errorMessages);
                    AddErrorMessages(key, errorMessages, errors);
                    if (errorMessages.Count == 0)
                    {
                        key.PendingChangeType = ChangeType.None;
                        key.KeyColumns.ForEach(column => column.PendingChangeType = ChangeType.None);
                    }
                    break;
            }
        }

        private void ImportDBForeignKeyChanges(DBForeignKey key, List<IBOError> errors)
        {
            if (!key.PendingChangeApproved) return;
            switch (key.PendingChangeType)
            {
                case ChangeType.Delete:
                    key.GetMappedDMRelationship().MarkForDelete();
                    key.MarkForDelete();
                    break;
                case ChangeType.Add:
                case ChangeType.Update:
                    IList<string> errorMessages;
                    _dbToDMRelationshipImporter.Import(key, out errorMessages);
                    AddErrorMessages(key, errorMessages, errors);
                    if (errorMessages.Count == 0)
                    {
                        key.PendingChangeType = ChangeType.None;
                        key.ForeignKeyColumns.ForEach(column => column.PendingChangeType = ChangeType.None);
                    }
                    break;
            }
        }
    }
}