using System;
using System.Collections.Generic;
using System.Linq;
using Firestarter.DB.Schema.BO;
using FireStarterModeller.BO;
using FireStarterModeller.Logic;
using Habanero.Base;
using Habanero.BO;
using Habanero.BO.ClassDefinition;

namespace Firestarter.DB_DomainModel.Logic
{
    public class DBToDMRelationshipImporter : IDBToDMRelationshipImporter
    {
        public DBToDMRelationshipImporter(IRelationshipNameGenerator relationshipNameGenerator, IDefClassFactory classFactory)
        {
//            FMDefClassFactory factory = new FMDefClassFactory();
            if (relationshipNameGenerator == null) throw new ArgumentNullException("relationshipNameGenerator");
            if (classFactory == null) throw new ArgumentNullException("classFactory");
            RelationshipNameGenerator = relationshipNameGenerator;
            ClassFactory = classFactory;
        }       
        public DBToDMRelationshipImporter(IRelationshipNameGenerator relationshipNameGenerator)
        {
            if (relationshipNameGenerator == null) throw new ArgumentNullException("relationshipNameGenerator");
            RelationshipNameGenerator = relationshipNameGenerator;
        }

        public IRelationshipNameGenerator RelationshipNameGenerator { get; private set; }
        private IDefClassFactory _classFactory;
        public IDefClassFactory ClassFactory
        {
            get { return _classFactory ?? (_classFactory = new FMDefClassFactory()); }
            set { _classFactory = value; }
        }

        public DMRelationship Import(DBForeignKey dbForeignKey, out IList<string> errorMessages)
        {
            if (dbForeignKey == null) throw new ArgumentNullException("dbForeignKey");
            errorMessages = new List<string>(4);
            var parentDMClass = dbForeignKey.ParentDMClass();
            var relatedDMClass = dbForeignKey.RelatedDMClass();
            string standardStringMessage = CreateStandardErrorMessage(dbForeignKey);
            if(parentDMClass == null || relatedDMClass == null)
            {
                string expectedMessage = standardStringMessage + " since it does not have a related or owner Class";
                errorMessages.Add(expectedMessage);
                return null;
            }
            Validate(dbForeignKey, errorMessages);


            DMRelationship dmRelationship = GetMappedRelationship(dbForeignKey);

            var foreignKeyColumnInfos = dbForeignKey.ForeignKeyColumns.ToArray();
            CreateRelationshipProperties(dmRelationship, foreignKeyColumnInfos);

            DeleteAllRelPropsWithNoFKColumn(((IDMRelationship) dmRelationship).RelationshipProperties.ToArray(), dbForeignKey.ForeignKeyColumns);

            if (String.IsNullOrEmpty(dmRelationship.RelationshipName))
            {
                dmRelationship.RelationshipName = RelationshipNameGenerator.CreateRelationshipName(dmRelationship);
                Console.WriteLine(string.Format("RelationshipCreated For {4} - {0} '{1}' '{2}' Related To '{3}'", 
                        dmRelationship.RelationshipID, dmRelationship.RelationshipName, dmRelationship.Cardinality, 
                        dmRelationship.RelatedClassName, dmRelationship.OwnerClassName));
            }
            return dmRelationship;
        }

        private void Validate(DBForeignKey dbForeignKey, IList<string> errorMessages)
        {
            string standardStringMessage = CreateStandardErrorMessage(dbForeignKey);
            if (dbForeignKey.ForeignKeyColumns.Count == 0)
            {
                string expectedMessage = standardStringMessage + " since it does not have any columns";
                errorMessages.Add(expectedMessage);
            }

            if (dbForeignKey.ParentDMClass() == null)
            {
                string expectedMessage = standardStringMessage + " since the owner Class cannot be determined";
                errorMessages.Add(expectedMessage);
            }
            if (dbForeignKey.RelatedDMClass() == null)
            {
                string expectedMessage = standardStringMessage + " since the related Class cannot be determined";
                errorMessages.Add(expectedMessage);
            }

            DBForeignKeyColumn[] foreignKeyColumnInfos = dbForeignKey.ForeignKeyColumns.ToArray();
            foreach (var foreignKeyColumn in foreignKeyColumnInfos)
            {
                ValidateFKColumns(foreignKeyColumn, errorMessages);
            }
        }

        public void DeleteAllRelPropsWithNoFKColumn(IEnumerable<IDMRelationshipProperty> relationshipProperties, IEnumerable<DBForeignKeyColumn> foreignKeyColumns)
        {
            IEnumerable<IDMRelationshipProperty> relPropsWithNoMatchingFKColumn = from relationshipProperty in relationshipProperties
                     let hasForeignKeyColumn = HasForeignKeyColumn(foreignKeyColumns, relationshipProperty)
                     where !hasForeignKeyColumn
                     select relationshipProperty;
            foreach (var relationshipProperty in relPropsWithNoMatchingFKColumn)
            {
                relationshipProperty.MarkForDelete();
            }
        }

        private bool HasForeignKeyColumn(IEnumerable<DBForeignKeyColumn> dbForeignKeyColumns, IDMRelationshipProperty relationshipProperty)
        {
            DBForeignKeyColumn foundFKColumn = FindForeignKeyColumn(dbForeignKeyColumns, relationshipProperty);
            return foundFKColumn != null;
        }

        public DBForeignKeyColumn FindForeignKeyColumn(IEnumerable<DBForeignKeyColumn> dbForeignKeyColumns, IDMRelationshipProperty relationshipProperty)
        {
            string ownerPropertyName = relationshipProperty.OwnerPropertyName;
            return dbForeignKeyColumns.FirstOrDefault(column => column.GetParentPropertyName() == ownerPropertyName 
                && column.GetRelatedPropertyName() == relationshipProperty.RelatedPropertyName);
        }

        public void CreateRelationshipProperties(DMRelationship dmRelationship, IEnumerable<DBForeignKeyColumn> foreignKeyColumnInfos)
        {
            foreach (DBForeignKeyColumn foreignKeyColumnInfo in foreignKeyColumnInfos)
            {
                DMRelationshipProperty relPropDef = CreateRelationshipProperty(foreignKeyColumnInfo);
                dmRelationship.RelationshipProperties.Add(relPropDef);
            }
        }

        public DMRelationshipProperty CreateRelationshipProperty(DBForeignKeyColumn foreignKeyColumnInfo)
        {
            var parentPropertyName = foreignKeyColumnInfo.GetParentPropertyName();
            var relatedPropertyName = foreignKeyColumnInfo.GetRelatedPropertyName();
            return ClassFactory.CreateRelPropDef(parentPropertyName, relatedPropertyName) as DMRelationshipProperty;
        }

        private static string CreateStandardErrorMessage(DBForeignKey dbForeignKey)
        {
            return string.Format
                ("A relationship could not be created for the foreign key called '{1}' on table '{0}'",
                 dbForeignKey.ParentTable, dbForeignKey.ForeignKeyName);
        }


        private static void ValidateFKColumns(DBForeignKeyColumn foreignKeyColumnInfo, IList<string> errorMessages)
        {
            string standardStringMessage = CreateStandardErrorMessage(foreignKeyColumnInfo.ForeignKey);
            if (foreignKeyColumnInfo.ParentColumn == null)
            {
                errorMessages.Add(CreateParentColumnNullMessage(standardStringMessage));
                return;
            }
            DMProperty property = foreignKeyColumnInfo.ParentColumn.GetMappedDMProperty();
            if (property == null)
            {
                errorMessages.Add(CreatePropertyNullMessage(standardStringMessage, foreignKeyColumnInfo));
                return;
            }

            if (property.Class == null)
            {
                errorMessages.Add(CreatePropertyClassNullMessage(standardStringMessage, foreignKeyColumnInfo));
                return;
            }
            DBColumn relatedColumn = foreignKeyColumnInfo.RelatedColumn;
            if (relatedColumn == null)
            {
                errorMessages.Add(CreateRelatedColumnNullMessage(standardStringMessage, foreignKeyColumnInfo));
                return;
            }
            DBTable relatedTable = relatedColumn.Table;
            if (relatedTable == null)
            {
                errorMessages.Add(CreateRelatedTableNullErrorMessage(standardStringMessage, foreignKeyColumnInfo));
                return;
            }
            var relatedClass = relatedTable.GetMappedDMClass();
            if (relatedClass == null)
            {
                errorMessages.Add(CreateRelatedClassNullErrorMessage(standardStringMessage, foreignKeyColumnInfo));
                return;
            }
            return;
        }

        public DMRelationship GetMappedRelationship(DBForeignKey dbForeignKey)
        {
            DMRelationship mappedDMRelationship = dbForeignKey.GetMappedDMRelationship();
            if (mappedDMRelationship == null)
            {
                mappedDMRelationship = ClassFactory.CreateSingleRelationshipDef("", "", "",null, false, DeleteParentAction.DoNothing,
                                                                                InsertParentAction.DoNothing, RelationshipType.Association) as DMRelationship;
                if (mappedDMRelationship == null) return null;
                mappedDMRelationship.MappedDBForeignKey = dbForeignKey;
                mappedDMRelationship.Cardinality = Cardinality.Single;
                mappedDMRelationship.DeleteAction = DeleteParentAction.DoNothing;
            }
            mappedDMRelationship.OwnerClass = dbForeignKey.ParentDMClass();
            var relatedDMClass = dbForeignKey.RelatedDMClass();
            if (relatedDMClass != null)
            {
                mappedDMRelationship.RelatedClassName = relatedDMClass.ClassNameBO;
                mappedDMRelationship.RelatedAssemblyName = relatedDMClass.AssemblyName;
            }
            return mappedDMRelationship;
        }

        private static string CreateParentColumnNullMessage(string strandardStringMessage)
        {
            return strandardStringMessage + string.Format(
                " since the foreign key Parent column is null");
        }
        private static string CreatePropertyNullMessage(string strandardStringMessage, DBForeignKeyColumn foreignKeyColumnInfo)
        {
            return strandardStringMessage + string.Format(
                " since the foreign key column '{0}' is not mapped to a domain model class property",
                foreignKeyColumnInfo.ParentColumnName);
        }

        private static string CreatePropertyClassNullMessage(string strandardStringMessage, DBForeignKeyColumn foreignKeyColumnInfo)
        {
            return strandardStringMessage + string.Format(
                " since the foreign key column '{0}' do not exist on the domain model class",
                foreignKeyColumnInfo.ParentColumnName);
        }

        private static string CreateRelatedClassNullErrorMessage
            (string strandardStringMessage, DBForeignKeyColumn foreignKeyColumnInfo)
        {
            return strandardStringMessage + string.Format(
                " since the related table '{0}' for column '{1}' is not mapped to a domain model class",
                foreignKeyColumnInfo.RelatedTableName, foreignKeyColumnInfo.RelatedColumnName);
        }

        private static string CreateRelatedTableNullErrorMessage
            (string strandardStringMessage, DBForeignKeyColumn foreignKeyColumnInfo)
        {
            return strandardStringMessage + string.Format(
                " since the table for the related column '{0}' does not exist",
                foreignKeyColumnInfo.RelatedColumnName);
        }

        private static string CreateRelatedColumnNullMessage(string strandardStringMessage, DBForeignKeyColumn foreignKeyColumnInfo)
        {
            return strandardStringMessage + string.Format(
                " since the column that relates to the foreign key column called '{0}' does not exist",
                foreignKeyColumnInfo.ParentColumnName);
        }
    }
}